#! /usr/bin/env python
#-----------------------------------------------------------------------------
# Copyright (c) 2021, 2022, Oracle and/or its affiliates.
#
# This software is dual-licensed to you under the Universal Permissive License
# (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl and Apache License
# 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose
# either license.
#
# If you elect to accept the software under the Apache License, Version 2.0,
# the following applies:
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#-----------------------------------------------------------------------------

"""
A script to help identify memory leaks.

The script argument should be the name of a file containing the trace output
when DPI_DEBUG_LEVEL is set to DPI_DEBUG_LEVEL_MEM (32):

    export DPI_DEBUG_LEVEL=32
    ./myprog >& mem.log
    python mem_leak.py mem.log
"""

import sys

ALLOCATED_STRING = ": allocated "
FREED_STRING = ": freed "

OCI_ALLOCATED_STRING = ": OCI allocated "
OCI_FREED_STRING = ": OCI freed "

def check_memory_use(file_name):
    """
    Checks memory use found in the given file. This file is assumed to have
    been generated by setting DPI_DEBUG_LEVEL to the value 32.
    """
    memory_locations = {}
    for line in open(file_name):

        # check for ODPI-C allocation
        pos = line.find(ALLOCATED_STRING)
        if pos > 0:
            parts = line[pos + len(ALLOCATED_STRING):].split()
            size = int(parts[0])
            memory_location = parts[3]
            memory_locations.setdefault(memory_location, []).append(size)
            continue

        # check for ODPI-C free
        pos = line.find(FREED_STRING)
        if pos > 0:
            parts = line[pos + len(FREED_STRING):].split()
            memory_location = parts[2]
            memory_locations[memory_location].pop(0)
            if not memory_locations[memory_location]:
                del memory_locations[memory_location]
            continue

        # check for OCI allocation
        pos = line.find(OCI_ALLOCATED_STRING)
        if pos > 0:
            parts = line[pos + len(OCI_ALLOCATED_STRING):].split()
            size = int(parts[0])
            memory_location = parts[3]
            memory_locations.setdefault(memory_location, []).append(size)
            continue

        # check for OCI free
        pos = line.find(OCI_FREED_STRING)
        if pos > 0:
            parts = line[pos + len(OCI_FREED_STRING):].split()
            memory_location = parts[2]
            memory_locations[memory_location].pop(0)
            if not memory_locations[memory_location]:
                del memory_locations[memory_location]
            continue

    print("Potential Memory Leaks")
    for memory_location in memory_locations:
        print(memory_location, "-> size",
              sum(memory_locations[memory_location]))

check_memory_use(sys.argv[1])



#------------------------------------------------------------------------------
# Copyright (c) 2023, Oracle and/or its affiliates.
#
# This software is dual-licensed to you under the Universal Permissive License
# (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl and Apache License
# 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose
# either license.
#
# If you elect to accept the software under the Apache License, Version 2.0,
# the following applies:
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# parameters_table.py
#
# Defines a directive (parameters-table) that creates a table with a specific
# configuration on top of the extended "list-table-with-summary" directive.
#------------------------------------------------------------------------------

from docutils.statemachine import ViewList

import table_with_summary

class ParametersTable(table_with_summary.ListTableWithSummary):

    def run(self):
        self.options["summary"] = \
            "The first column displays the name of the parameter. The " \
            "second column displays the parameter's mode. The third column " \
            "displays the description of the parameter."
        self.options["header-rows"] = 1
        self.options["class"] = ["wy-table-responsive"]
        self.options["widths"] = [10, 5, 35]
        headings = ViewList(['* - Parameter', '  - Mode', '  - Description'])
        self.content = headings + self.content
        return super().run()

def setup(app):
    app.add_directive('parameters-table', ParametersTable)



#------------------------------------------------------------------------------
# Copyright (c) 2022, Oracle and/or its affiliates.
#
# This software is dual-licensed to you under the Universal Permissive License
# (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl and Apache License
# 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose
# either license.
#
# If you elect to accept the software under the Apache License, Version 2.0,
# the following applies:
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# table_with_summary.py
#
# Defines a directive (list-table-with-summary) that adds support for a summary
# option on top of the regular "list-table" directive.
#------------------------------------------------------------------------------

import sphinx
from docutils.parsers.rst import directives
from docutils.parsers.rst.directives import tables

from packaging import version

class ListTableWithSummary(tables.ListTable):

    option_spec = {'summary': directives.unchanged}
    option_spec.update(tables.ListTable.option_spec)

    def run(self):
        result = super().run()
        summary = self.options.get('summary')
        if summary:
            table_node = result[0]
            table_node["summary"] = summary
        return result


class HTMLTranslator(sphinx.writers.html5.HTML5Translator):

    def visit_paragraph(self, node):
        children = node.children
        if len(children) == 1 and children[0].astext() == "No relevant notes":
            atts = {"style": "clip: rect(1px, 1px, 1px, 1px);"
                             "clip-path: inset(50%);"
                             "height: 1px;"
                             "overflow: hidden;"
                             "position: absolute;"
                             "white-space: nowrap;"
                             "width: 1px;"
                   }
            self.body.append(self.starttag(node, 'p', '', **atts))
        else:
            super().visit_paragraph(node)

    def visit_table(self, node):
        if version.parse(sphinx.__version__) > version.parse('4.2.0'):
            self._table_row_indices = [0]
        else:
            self._table_row_index = 0

        atts = {}
        classes = [cls.strip(' \t\n') \
                for cls in self.settings.table_style.split(',')]
        classes.insert(0, "docutils")  # compat

        # set align-default if align not specified to give a default style
        classes.append('align-%s' % node.get('align', 'default'))

        if 'width' in node:
            atts['style'] = 'width: %s' % node['width']
        if 'summary' in node:
            atts['summary'] = node['summary']
        tag = self.starttag(node, 'table', CLASS=' '.join(classes), **atts)
        self.body.append(tag)


def setup(app):
    app.add_directive('list-table-with-summary', ListTableWithSummary)
    app.set_translator("html", HTMLTranslator, override=True)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#------------------------------------------------------------------------------
# Copyright (c) 2016, 2025, Oracle and/or its affiliates.
#
# This software is dual-licensed to you under the Universal Permissive License
# (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl and Apache License
# 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose
# either license.
#
# If you elect to accept the software under the Apache License, Version 2.0,
# the following applies:
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

import os
import sys

# If your extensions are in another directory, add it here.
sys.path.append(os.path.abspath("_ext"))

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ["table_with_summary", "parameters_table", 'sphinx_rtd_theme']

# the location of templates that are being used, relative to this directory
templates_path = ['.templates']

# the suffix used for all source files
source_suffix = '.rst'

# the name of the root document
root_doc = master_doc = 'index'

# general information about the project
project = 'ODPI-C'
copyright = '2016, 2025, Oracle and/or its affiliates. All rights reserved.'
author = 'Oracle'

# the version info for the project, acts as replacement for |version| and
# |release|, also used in various other places throughout the built documents
#
# the short X.Y version
version = '5.6'

# the full version, including alpha/beta/rc tags
release = '5.6.3'

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# Options for HTML output
# -----------------------

# The style sheet to use for HTML and HTML Help pages. A file of that name
# must exist either in Sphinx' static/ path, or in one of the custom paths
# given in html_static_path.
# html_style = 'default.css'

# The theme to use for readthedocs.
html_theme = 'sphinx_rtd_theme'

# the name for this set of documents.
html_title = 'ODPI-C v' + release

# the location for static files (such as style sheets) relative to this
# directory; these are copied after the builtin static files and will overwrite
# them
html_static_path = ['.static']

# the location of the favicon to use for all pages
# html_favicon = "_themes/oracle/static/favicon.ico"

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
html_last_updated_fmt = '%b %d, %Y'

# If true, the reST sources are included in the HTML build as _sources/<name>.
html_copy_source = False

# Output file base name for HTML help builder.
htmlhelp_basename = 'oracledbdoc'

numfig = True

# default domain is C
primary_domain = "c"

# Display tables with no horizontal scrollbar
def setup(app):
    app.add_css_file('custom.css')
